package dsnserver;
import java.util.*;

/**********************************************
 * The Parser Class is the main classe of the 
 * parser classes.  The parsers have to initialised here.  
 * In the main function we will read the lines from 
 * the node and send the lines to the  
 * parser it belongs to.
 *   
 * @author  kaltt
 *********************************************/
public class Parser extends Thread {
	
	Communication myComm;
	CmdParser pTime = new DSNtimetoTime(myComm);
	CmdParser pDSNID = new CmdParserDSNID();
	CmdParser pDSNVersion = new CmdParserDSNVersion();
	CmdParser pDSNLocation = new CmdParserDSNLocation();
	CmdParser pLogging = new CmdParserLogging((DSNtimetoTime)pTime);
	CmdParser pDSNProgInfo = new CmdParserDSNProgInfo();
	CmdParser pGetAddr = new CmdParserGetAddr();
	CmdParser pTargetCommand = new CmdParserTargetCommand();
	CmdParser pLoadFile = new CmdParserLoadFile();
	CmdParser pTargetFlash = new CmdParserTargetFlash();
	CmdParser pConinfo = new CmdParserConinfo();
	CmdParser pRPCS = new CmdParserRPCS();
	
	
	//private static String parserState = "";
	private static CmdParser parserState = null;
	public static Map cmdParserPattern;

	public Parser(Communication comm){
		myComm = comm;
		cmdParserPattern = new HashMap();
		init();
	}
	
	/**********************************************
	 * All CmdParser has to be initialized here
	 * by using the function addCmdParser(key, value).
	 * Key must be the string which we are looking for 
	 * and Value is the name of the Method which 
	 * has to be used in this case.
	 *********************************************/
	private void init(){
		addCmdParser(":DP", pDSNID);
		addCmdParser(":DV", pDSNVersion);
		addCmdParser(":DL", pDSNLocation);
		addCmdParser(":SL", pLogging);
		addCmdParser(":DT", pTime);
		addCmdParser(":XP", pDSNProgInfo);
		addCmdParser("Local", pGetAddr);
		//addCmdParser(":DC", pTargetCommand);
		addCmdParser("ready", pLoadFile);
		addCmdParser(":LH", pLoadFile);
		//addCmdParser(":TP", pTargetFlash);
		addCmdParser(":TF", pTargetFlash);
		addCmdParser(":T", pConinfo);
		addCmdParser(":RP", pRPCS);
	}
	

	/**********************************************
	 * Key is the String we are looking for at 
	 * the beginning of the line and Value is the 
	 * name of the Method which has to be used if 
	 * the key is found at the beginning of the line.
	 *********************************************/
	@SuppressWarnings("unchecked")
	private void addCmdParser(String key, CmdParser value){
		cmdParserPattern.put(key, value);
	}

	/**********************************************
	 * To reset the variable parserState.
	 * Is used by the cmdParser to say the Parser 
	 * that he is not expecting another line.
	 *********************************************/	
	public static void resetParserState(){
		parserState = null;
	}
	
	/**********************************************
	 * To set the variable parserState. 
	 * Is used by the cmdParser to say the Parser 
	 * that the next line is for the same  cmdParser 
	 * as well.
	 * 
	 *********************************************/	
	public static void setParserState(CmdParser state){
		parserState = state;
	}
	
	
	/**********************************************
	 * To get the variable parserState.
	 * @uml.property  name="parserState"
	 *********************************************/	
	public static CmdParser getParserState(){
		return parserState;
	}
	
	public void run() {
		
		String line;
		String[] token;
		
		while(true){
			line = myComm.readLine();
//System.out.print(line);			
			//eliminates [jaws@xx:xy] if it appears somewhere 
			line=line.replaceAll("\\[jaws@[^]]*\\]\\$", "");
			//eliminates [jaws@xx:xy] if it appears at the beginning of a line
//			if (line.startsWith("[jaws@")){
//				line = line.substring(13);
//			}
			token = line.split(" ");
			if (parserState != null){
				parserState.parseLine(line);
			}
			else if (cmdParserPattern.containsKey(token[0])){
				//long start = System.currentTimeMillis();
				((CmdParser)cmdParserPattern.get(token[0])).parseLine(line);
				//if((System.currentTimeMillis() - start)>50){ System.out.println((System.currentTimeMillis() - start) + "ms  Line: "+line );}
			}
			else if (line.equals("\n")){}
			else{
				//System.out.print("unparsed line: " + line);
			}
		}
	}
	
	
}
