package dsnserver;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.xmlrpc.AuthenticatedXmlRpcHandler;
import org.apache.xmlrpc.WebServer;

/**********************************************
 * A function wich is used as a XmlRpc function.
 * It sends a .hex file to the BTnode. The hex.
 * file has to be in the folder /hex.
 *
 * @author kaltt
 *
 **********************************************/
public class XmlRpcLoadFile implements AuthenticatedXmlRpcHandler{

	WebServer server;
	Communication myComm;
	static boolean ready = false;
	static boolean transmission_ok = false;
	static boolean transmission_failed = false;
	static String transmission_err = ""; 
	
	public XmlRpcLoadFile(Communication comm, WebServer server) {
		this.server = server;
		myComm = comm;
	}

	
	/**********************************************
	 * For setting the variable ready. It is used 
	 * of the class CmdParserLoadFile and is set 
	 * when the sentense "ready to receive hexfile, press enter to quit."
	 * is given out after the BTnode command loadhex.
	 **********************************************/
	public static void setReady(){
		ready = true;
	}

	/**********************************************
	 * For setting the variable transmission_ok and 
	 * transmission_err. It is used of the class 
	 * CmdParserLoadFile and is set when everything 
	 * was seccessfull after the file was load up 
	 * to the BTnode. (Is set if ":LH completed: ..."
	 * was written out by the BTnode)
	 **********************************************/
	public static void setTransmission_ok(String line){
		transmission_ok = true;
		transmission_err = line;
	}
	
	/**********************************************
	 * For setting the variable transmission_failed and 
	 * transmission_err. It is used of the class 
	 * CmdParserLoadFile and is set when a error
	 * occured during the file upload to the BTnode. 
	 * (Is set if ":LH failed: ..." was written 
	 * out by the BTnode)
	 **********************************************/
	public static void setTransmission_failed(String line){
		transmission_failed = true;
		transmission_err = line;
	}
	
	/**********************************************
	 * It is the function which is called by the 
	 * XmlRpcServer. It is calling the function
	 * loadFileLocal.
	 * 
	 * @param String filename the name of the file 
	 * you will upload. <br>
	 * String type  0=dsn, 1=tg
	 * 
	 * @return Vector result Key = "err" 
	 * Value = the value of transmission_err 
	 **********************************************/
	public Vector loadFile(String filename, String type) {
		
		Vector result = new Vector();
		
		result = (Vector)loadFileLocal(filename, type);
		
		return result;		
	}

	
	/**********************************************
	 * It starts the upload of a .hex file which is
	 * in the folder /hex. 
	 * (if there occures a read error you have change 
	 * something in the loop "while(!transmission_failed && ((line = input.readLine()) != null)){")
	 * 
	 * 
	 * @param String filename the name of the file 
	 * you will upload to the GUI node. <br>
	 * String type  0=dsn, 1=tg
	 * 
	 * @return Vector result Key = "err" 
	 * Value = the value of transmission_err 
	 **********************************************/
	private Vector loadFileLocal(String filename, String type) {

		Vector<Hashtable> result = new Vector<Hashtable>();
		Hashtable<String,String> t;
		
		long uploadTime=0;
			
		File file = new File(GetProperty.get("hexPath"), filename);

		int tries=5;
		while(tries > 0 ) {

			FileReader f = null;
			try {
				f = new FileReader(file);
			} catch (FileNotFoundException e) {
				try {
					RXTXComm.writeLog("error (f = new FileReader(file)): " + e.toString());
				} catch (IOException e1) {}
			}
			BufferedReader input = new BufferedReader (f);

			String line = "";
			Date now = new Date();	    
			SimpleDateFormat df = new SimpleDateFormat( "yyMMddHHmm" );
			String time = df.format(now);
			
			myComm.lock();
			//That during this time a rest of the logs 
			//which were requested before the loadhex 
			//was started can be received from the GUI node.
			//Otherwise the GUI node is overloaded with 
			//reseiving logs and the hex-file and the transmission
			//will fail. 
			try {
				Thread.sleep(2000);
			} catch (InterruptedException e2) {
				// TODO Auto-generated catch block
				e2.printStackTrace();
			}
			ready = false;
			transmission_failed = false;
			myComm.writeLine("loadhex " + time + " " + type
				+ " " + filename.substring(0, Math.min(100,filename.length())));
			long mytime = System.currentTimeMillis();
			while (!ready && (System.currentTimeMillis()-mytime)<10000) {
				try {
					Thread.sleep(50);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			if (!ready) {
				transmission_failed=true;
				System.out.println("Loadfile: timeout waiting for LH prompt");
				myComm.writeLine(""); // abort intentionally wrong command
			}
			
			long c = 0;
			long c_total = file.length();
	
			System.out.println("");		
			System.out.println("LoadFile");		
			long startTime=System.currentTimeMillis();
			
			try {
			
				while(!transmission_failed && ((line = input.readLine()) != null)){			

				/*****************************************
				 * here we wait x milliseconds after every 
				 * line we send. This raises the time for 
				 * sending but there will occure less read 
				 * errors.
				 *****************************************/
				//System.out.println("wait");
				Thread.sleep(12);
				
					myComm.writeLine(line);
				
					c = c + line.length();
					if (c > (c_total/50)){
						System.out.print("|");
						c = c % (c_total/50);
					}
				}
			}
			catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			long mytime2 = System.currentTimeMillis();
			while (!transmission_failed && !transmission_ok &&(System.currentTimeMillis()-mytime2)<5000);
			if (!transmission_ok && !transmission_failed) {
				transmission_failed=true;
				System.out.println("Loadfile: timeout waiting for acknowledged LH");
			}
			
			myComm.unlock();			
			if (transmission_failed) {
				System.out.println("Loadfile: transmission failed, try again (" +tries+")" );
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				tries--;
			}
			else {
				tries=0;
				uploadTime = System.currentTimeMillis() - startTime;
			}
			try {
				input.close ();
			} catch (IOException e) {
				try {
					RXTXComm.writeLog("error (input.close()): " + e.toString());
				} catch (IOException e1) {}
			}
			try {
				f.close();
			} catch (IOException e) {
				try {
					RXTXComm.writeLog("error (f.close): " + e.toString());
				} catch (IOException e1) {}
			}
		}
		
		if (transmission_ok) {
			System.out.print("100% ("+uploadTime+" ms)");
			System.out.println("");
			myComm.lock();
			myComm.writeLine("cdist run");
			myComm.unlock();
			// this one blocks the rpc server unneccessarily
			// waitmillis(15000);
		}
		myComm.lock();
		myComm.writeLine("rpc 0 xbank get proginfo");
		myComm.unlock();

//while (transmission_err != ""){waitmillis(500);}
//System.out.println("transmission_err: "+transmission_err);		

		t = new Hashtable<String,String>();
		t.put("err", transmission_err);
		result.add(t);
		
		return result;
	}

	public Object execute(String method, Vector v, String user,  String password) {
		Object result = new Vector<Hashtable>();
		if(checkPassword(user, password)){			
			result = execute(method, v);
			return result;
		}
		else{
			try {
				RXTXComm.writeLog("Error in LoadFile: Unauthorized User (user:" + user + ")");
			} catch (IOException e1) {}
			return result;
		}
	}

	protected Object execute(String method, Vector v){
		Vector result = null;
		if(method.endsWith(".loadFile")){
			try {
				result = loadFileLocal((String)v.get(0),(String)v.get(1));
			} catch (NumberFormatException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return result;
	}
	
	public boolean checkPassword(String user, String password){
		boolean pwd = false;
		SQLDBcommunication.lock();
		ResultSet result = SQLDBcommunication.getResultsFromDB("" +
				"SELECT *  FROM `dsnpwd` " +
				"WHERE `name` = '" + user + "' " +
				"AND `pwd` = md5('" + password + "') " +
				"AND (`function` =  'loadFile' OR `function` =  'all' )");
		try {
			if (result.next()){pwd = true;}
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		SQLDBcommunication.unlock();
		if (pwd) return true;
		else return false;
	}

	
}

