package dsnserver;

import dsnserver.Communication;
import org.apache.xmlrpc.*;

/**********************************************
 * This is the XmlRpc server which starts a 
 * webserver that is listenig on a port which 
 * we can define. 
 * All function we will use with the XmlRpc 
 * have to be added to the server with
 * server.addHandler().  
 * 
 * The XmlRpcServer has to be started in a
 * main function. it has to look lake that:
 * <code>
 * XmlRpcServer server = 
 * new XmlRpcServer(Integer.parseInt(GetProperty.get("xmlRpcPort")));<br>
 * server.run();
 * </code>
 *  
 * @author kaltt
 *
 **********************************************/
public class XmlRpcServer {
    // the xml-rpc webserver
    static WebServer server;
    static int port=0;
    Communication myComm;
    
    @SuppressWarnings("static-access")
	public XmlRpcServer(int port, Communication comm){
    	this.port = port;
    	this.myComm = comm;
    }
    
    /**
     * main method
     */
    public void run () throws Exception {
	if (port < 1024 || port > 65535) {
	    System.err.println ("Usage: java Server port ");
	    System.out.println ("Range of Port: 1024 > port > 65535 ");
	    System.exit (1);
	}
	//XmlRpc.setDebug(true);
	server = new WebServer (port);
	// add an instance of this class as default handler
	server.addHandler ("getDSN", new XmlRpcGetDSN());
	server.addHandler ("setDSNLocation", new XmlRpcSetDSNLocation(myComm, server));
	server.addHandler ("getDBEvent", new XmlRpcGetDBEvent());
	server.addHandler ("targetCommand", new XmlRpcTargetCommand(myComm, server));
	server.addHandler ("targetFlash", new XmlRpcTargetFlash(myComm, server));
	server.addHandler ("dsnCommand", new XmlRpcDSNCommand(myComm, server));
	server.addHandler ("uploadFile", new XmlRpcUploadFile(server));
	server.addHandler ("getFileList", new XmlRpcGetFileList());
	server.addHandler ("loadFile", new XmlRpcLoadFile(myComm, server));
	server.addHandler ("dsnVerbosity", new XmlRpcDSNVerbosity(myComm, server));
	server.addHandler ("getConinfo", new XmlRpcGetConinfo());
	server.addHandler ("getServerTime", new XmlRpcGetServerTime());
	server.addHandler ("user", new XmlRpcUser());
	server.addHandler ("threadSwitch", new XmlRpcThreadSwitch());
	server.addHandler ("getRPCS", new XmlRpcGetRPCS());
	server.addHandler ("setNodeInfo", new XmlRpcSetNodeInfo());
	server.start (); 
	System.out.println ("Listening on port "+port);
    }
}
