package testserver;

import java.sql.SQLException;
import java.util.concurrent.Semaphore;

import dsnserver.CmdParserGetAddr;
import dsnserver.CmdParserLogging;
import dsnserver.Communication;
import dsnserver.DSNtimetoTime;
import dsnserver.GetProperty;
import dsnserver.Parser;
import dsnserver.RXTXComm;
import dsnserver.SQLDBcommunication;
import dsnserver.ThreadCommand;
import dsnserver.ThreadLogging;
import dsnserver.XmlRpcServer;
import dsnserver.XmlRpcThreadSwitch;
import dsnserver.CmdParserLogging.dsn;

/** 
 * @author  kaltt
 */
public class DSNServer_TEST {

	private Communication myComm;
	public static Semaphore threadSemaphore = new Semaphore(1, true);

	public DSNServer_TEST(){

	}
	
	protected void finalize() {
		try {
			SQLDBcommunication.closeConnectionToDB();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) {
		DSNServer_TEST dsnTest = new DSNServer_TEST();
		dsnTest.run();
	}	
	
	private void wait(int mm){
		try {
			Thread.sleep(mm);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private void run(){	
		
		myComm = new RXTXComm();
//		myComm = new TestComm();
		myComm.open("test");
	
	
		try {
			SQLDBcommunication.connectToDB();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("");
		System.out.println("Create all tables if they don't exist.");
		SQLDBcommunication.makeTable();
		System.out.println("");

		if (GetProperty.get("truncate_Table").equals("yes")){
			System.out.println("Delete all tables.");
			System.out.println("");
			SQLDBcommunication.sendQueryToDB("TRUNCATE TABLE `dsninfo`"); 
			SQLDBcommunication.sendQueryToDB("TRUNCATE TABLE `dsnlog`"); 
			SQLDBcommunication.sendQueryToDB("TRUNCATE TABLE `dsnconinfo`"); 			
			SQLDBcommunication.sendQueryToDB("TRUNCATE TABLE `dsnrpcs`"); 
		}
		
		if (GetProperty.get("deletLog").equals("yes")&&GetProperty.get("logFile").equals("yes")){
			System.out.println("Delete the log file.");
			System.out.println("");
			myComm.deleteLog();
		}
		
		System.out.println("Start Parser.");
		Parser parser = new Parser(myComm);
		System.out.println("Thread-Parser: " + parser.getName());
		parser.start();
		System.out.println("");
		
		if (GetProperty.get("reboot_node").equals("yes")){		
			System.out.println("Reset all nodes.");
			System.out.println("");
			myComm.writeLine("rpcs 0 reset");		
			wait(8000);
		}
		

		if (GetProperty.get("JawsTime").equals("yes")){			
			myComm.writeLine("dsn get time");
			wait(20000);
		}
		
		System.out.println("Get DSNnode addr.");
		myComm.writeLine("bt addr");
		wait(500);		
		int i = 0;
		while (CmdParserGetAddr.DSNnodeAddr=="00:00:00:00:00:00" && i<5){
			i++;
			wait(1000);
			myComm.writeLine("bt addr");
		}
		if (CmdParserGetAddr.DSNnodeAddr=="00:00:00:00:00:00"){
			System.out.println("GUInode is not responding!");
			System.exit(0);
		}
		System.out.println("");
				
		if(GetProperty.get("clearLogBTnode").equals("yes")){
			System.out.println("Clear log.");
			myComm.writeLine("rpc 0 log clear");
			wait(500);
			System.out.println("");
		}
			
		//String of ThreadCommand must be the same as in 
		//XmlRpcThreadSwitch for starting and stopping
		//this thread.
		if(GetProperty.get("DSNPing").equals("yes")){XmlRpcThreadSwitch.DSNPingSwitch = true;}
		else {XmlRpcThreadSwitch.DSNPingSwitch = false;}
		System.out.println("Start TreadCommand (dsn ping).");
		Thread ping = new ThreadCommand(myComm, Integer.parseInt(GetProperty.get("getDSN_Ping"))
				, "rpc 0 dsn ping");
		System.out.println("Thread-Ping: " + ping.getName());
		System.out.println("");
		ping.start();

		wait(500);

		//String of ThreadCommand must be the same as in 
		//XmlRpcThreadSwitch for starting and stopping
		//this thread.
		if(GetProperty.get("DSNVersion").equals("yes")){XmlRpcThreadSwitch.DSNVersionSwitch = true;}
		else {XmlRpcThreadSwitch.DSNVersionSwitch = false;}
		System.out.println("Start TreadCommand (dsn Version).");
		Thread version = new ThreadCommand(myComm, Integer.parseInt(GetProperty.get("getDSN_Version"))
				, "rpc 0 dsn get version");
		System.out.println("Thread-Version: " + version.getName());
		System.out.println("");
		version.start();
		
		wait(500);
		
		//String of ThreadCommand must be the same as in 
		//XmlRpcThreadSwitch for starting and stopping
		//this thread.
		if(GetProperty.get("DSNLocation").equals("yes")){XmlRpcThreadSwitch.DSNLocationSwitch = true;}
		else {XmlRpcThreadSwitch.DSNLocationSwitch = false;}
		System.out.println("Start TreadCommand (dsn location).");
		Thread location = new ThreadCommand(myComm, Integer.parseInt(GetProperty.get("getDSN_Location"))
				, "rpc 0 dsn get loc");
		System.out.println("Thread-Location: " + location.getName());
		System.out.println("");
		location.start();
		
		wait(500);

		//String of ThreadCommand must be the same as in 
		//XmlRpcThreadSwitch for starting and stopping
		//this thread.		
		if(GetProperty.get("DSNProgInfo").equals("yes")){XmlRpcThreadSwitch.DSNProgInfoSwitch = true;}
		else {XmlRpcThreadSwitch.DSNProgInfoSwitch = false;}
		System.out.println("Start TreadCommand (xbank get proginfo).");
		Thread progInfo = new ThreadCommand(myComm, Integer.parseInt(GetProperty.get("getDSN_ProgInfo"))
				, "rpc 0 xbank get proginfo");
		System.out.println("Thread-ProgInfo: " + progInfo.getName());
		System.out.println("");
		progInfo.start();

		wait(500);
		
		//String of ThreadCommand must be the same as in 
		//XmlRpcThreadSwitch for starting and stopping
		//this thread.
		if(GetProperty.get("Coninfo").equals("yes")){XmlRpcThreadSwitch.ConinfoSwitch = true;}
		else {XmlRpcThreadSwitch.ConinfoSwitch = false;}
		System.out.println("Start TreadCommand (coninfo).");
		Thread coninfo = new ThreadCommand(myComm, Integer.parseInt(GetProperty.get("getDSN_coninfo"))
				, "coninfo");
		System.out.println("Thread-Coninfo: " + coninfo.getName());
		System.out.println("");
		coninfo.start();

		wait(500);
		
		if(GetProperty.get("JawsTime").equals("yes")){XmlRpcThreadSwitch.JawsTimeSwitch = true;}
		else {XmlRpcThreadSwitch.JawsTimeSwitch = false;}
		System.out.println("Start time function.");
		DSNtimetoTime time = new DSNtimetoTime(myComm);
		System.out.println("Thread-Time: " + time.getName());
		System.out.println("");
		time.start();

		wait(500);
		
		System.out.println("Start XmlRpcServer.");
		try {
			XmlRpcServer server = new XmlRpcServer(Integer.parseInt(GetProperty.get("xmlRpcPort")), myComm);
			server.run();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		wait(500);
		if(GetProperty.get("LogThread").equals("yes")){XmlRpcThreadSwitch.LoggingSwitch = true;}
		else {XmlRpcThreadSwitch.LoggingSwitch = false;}
		System.out.println("");
		System.out.println("Start TreadLogging.");
		ThreadLogging logging = new ThreadLogging(myComm);
		System.out.println("Thread-Logging: " + logging.getName());
		System.out.println("");
		logging.start();
						
	}		
	
}
